#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "filedat.h"
#include "index.h"

#define EXTLEN 4
#define KEYNUM 3
#define KEYLEN 100
#define NODENUMS (KEYNUM + 1)
#define NAMELEN 100


typedef struct key_type {
	char key[KEYLEN];
	long recno;
	} KEYREC;

typedef struct noderec_type {
	int numkeys;
	struct key_type keyrecs[KEYNUM];
	long nodenums[NODENUMS];
	} NODEREC;

 static int indexfid;
 static long root_rec = 0L;
 
 static void printorder(long);
 static void printnodes(long);
 static long addnode(long, char*, long);
 static long delnode( long nodeno, char *key);


void openindex(char *filename)
	{
	 char iname[NAMELEN + EXTLEN];
	 NODEREC node;
	
	strcpy(iname, filename);
	strcat(iname, ".idx");
	
	indexfid =  openfile(iname, sizeof(NODEREC));
	
	if( getnextrecno(indexfid) == 0L)
	   root_rec = 0L;
	   else
	   root_rec = 1L;
	}

void closeindex(void)
	{
	closefile (indexfid);
	}


 void print_rec_names (void)
	{
	if(root_rec != 0)
	printorder(root_rec);  
	}

static void printorder(long nodeno)
	{
	NODEREC node;
	int i;

	readrec((char*)&node, nodeno, indexfid);

	for(i = 0; i < node.numkeys ; i++)
		{
		if(node.nodenums[i] != 0)
			printorder(node.nodenums[i] );
		get_rec(node.keyrecs[i].recno);
		}
		
	 if(node.nodenums[i] != 0)
		printorder(node.nodenums[i] );
	 }
	

 void printindex(void)
	{
	if(root_rec != 0)
	printnodes(root_rec); 
	}


static void printnodes(long nodeno)
	{
	NODEREC node;
	int i, len;

	readrec((char*)&node,nodeno,indexfid);

	printf("\nNode %ld, Numkeys = %d\n", nodeno, node.numkeys);

	if(node.numkeys != 0)
		{
		printf("Keys & Recnos");
		for(i = 0; i < node.numkeys ; i++)
	    	{
	    	printf("   %s %-3ld", node.keyrecs[i].key, node.keyrecs[i].recno);
	     	}
		printf("\nNodenums     ");
		for(i = 0; i < node.numkeys ; i++)
	    	{
	     	printf("   %-3ld%*s ", node.nodenums[i], (int) strlen(node.keyrecs[i].key), " " );
	     	}
	     	printf(" %-3ld\n", node.nodenums[i]);

		for(i = 0; i <= node.numkeys ; i++)
	    	{
	     	if(node.nodenums[i] != 0)
	        	printnodes(node.nodenums[i] );
	     	}
		}
	}
	

 void addindex(char *key, int recno)
	{
	long resno;
	
	resno = addnode(root_rec, key, recno);
	if(resno == 1)
		root_rec = resno;
	}

 static long addnode(long nodeno, char *key, long recno)
	{
	long resno, nextrecno;
	int k,i;
	NODEREC node;

	if(nodeno == 0)
		{
		 node.numkeys = 1;
		 node.nodenums[0] = 0;
		 node.nodenums[1] = 0;
		 node.keyrecs[0].recno = recno;
		 strcpy(node.keyrecs[0].key, key);
		 
		nextrecno = setnextrecno(indexfid);
		writerec((char*)&node,nextrecno, indexfid);
		return nextrecno;
		}
     else
       {
		readrec((char*)&node,nodeno,indexfid);

		for(i = 0; i < node.numkeys && (strcmp( node.keyrecs[i].key, key) < 0) ; i++);

	
		if( node.numkeys < KEYNUM)  /* If room add into this recnode */
			{
		  	node.nodenums[node.numkeys + 1] = node.nodenums[node.numkeys];
			if (i < node.numkeys)
				{
		 	    for(k = node.numkeys ; k >= i; k--)
		 		     {
		  		      node.keyrecs[k] = node.keyrecs[k - 1];
		  		      node.nodenums[k] = node.nodenums[k - 1];
		    	      }
		    	 }
		     
		 	node.numkeys++;
			node.nodenums[i] = 0;
		 	node.keyrecs[i].recno = recno;
			strcpy(node.keyrecs[i].key, key);
		 
			writerec((char*)&node,nodeno,indexfid);
			}
     		else  /* if no room, then go down to next level until you find a node with room */
       	 		{
      	 		 resno =  addnode(node.nodenums[i],key,recno);
      	  		 if(resno != 0L)
      	  			{
      	  			node.nodenums[i] = resno;
					writerec((char*)&node,nodeno,indexfid);
					}
       	  		}    
       	return 0L;
        }
 	}

 void delindex(char *key)
	{
	
	printf("Delindex and delete functions needed\n");

	}


void print_index_file(void)
	{
	NODEREC node;
	int i;
	long k, nextrecno;
	
	nextrecno = getnextrecno(indexfid);

		for (k = 1; k <= nextrecno ; k++)
		{
		readrec((char*)&node,k,indexfid);

		printf("\n\tNode %ld, Numkeys = %d\n", k, node.numkeys);
		printf("\tNodenums    Keys and Record Numbers\n");

		for(i = 0; i < node.numkeys ; i++)
			{
			printf("\t  %3ld       ", node.nodenums[i]);
			printf("%-8s ", node.keyrecs[i].key);
			printf(" %ld\n", node.keyrecs[i].recno);
			}
		if(node.numkeys != 0)
			printf("\t  %3ld\n", node.nodenums[i]);
		}
	 }
	

